Portfolio Manager Demo Using DSE Spark
---------------------------------------

This demo simulates a financial portfolio manager application. Sample portfolio
data is generated, including a list of stocks, the number of shares purchased,
and the purchase price. The demo's pricer utility simulates real-time stock data
where each portfolio updates based on its overall value and the percentage of
gain or loss compared to the purchase price. This utility also generates 100
days of historical market data (the end-of-day price) for each stock. Finally,
a Spark job calculates the greatest historical 10-day loss period for
each portfolio.

For more detailed instructions including screenshots of the sample Portfolio
web application, see the DataStax documentation website:
http://docs.datastax.com/en/getting_started/doc/getting_started/demos.html


Steps:

1. Start a DSE cluster with Spark enabled as Analytics nodes

2. Generate sample stock data for the application:

        $ cd install_location/demos/portfolio_manager
        $ bin/pricer --help
        $ bin/pricer -o INSERT_PRICES
        $ bin/pricer -o UPDATE_PORTFOLIOS
        $ bin/pricer -o INSERT_HISTORICAL_PRICES -n 100

   The pricer utility may take a couple minutes to run.

3. Start the web service:

        $ cd website
        $ sudo ./start

   Open a browser and go to http://localhost:8983/portfolio.

   The real-time Portfolio Manager demo application is displayed. Note that Historical
   Loss values will show as "?" because they have not yet been calculated.

4. Run the Spark Demo application (set dse_install_location/bin in PATH):

        $ dse spark-sql -f 10_day_loss.q

5. The same processing steps are avaiable in a Spark scala example:

        $ cd install_location/demos/portfolio_manager
        $ ./10-day-loss.sh

6. The same processing steps are available in a Spark Java example:

        $ cd install_location/demos/portfolio_manager
        $ ./10-day-loss-java.sh

7. After the job completes, refresh the Portfolio Manager web page. The results of
   the Largest Historical 10 day Loss for each portfolio are displayed.

Source code for both demos can be found under install_location/demos/portfolio_manager/src.

For more information on the Portfolio Manager Demo, go to:
http://docs.datastax.com/en/getting_started/doc/getting_started/demos.html
