/*
 * Copyright DataStax, Inc.
 *
 * Please see the included license file for details.
 */

package com.datastax.dse.demos.solr.load;

import java.io.File;
import java.io.PrintStream;
import java.util.Iterator;
import java.util.List;
import java.util.Random;
import java.util.concurrent.Callable;
import java.util.concurrent.TimeUnit;

import com.google.common.base.Suppliers;
import com.google.common.collect.ImmutableList;
import com.google.common.collect.ImmutableMap;
import com.google.common.collect.ImmutableSet;
import org.junit.After;
import org.junit.Before;
import org.junit.BeforeClass;
import org.junit.Test;

import static org.junit.Assert.assertEquals;

import com.datastax.dse.demos.solr.CommandLine;
import com.datastax.dse.demos.solr.SolrStress;
import com.datastax.dse.demos.solr.clients.Clients;
import com.datastax.dse.demos.solr.clients.MockSolrClientFactory;
import com.datastax.dse.demos.solr.stats.Metrics;

public class HTTPLoadStrategyTest
{
    static final int THREAD_COUNT = 4;
    static MockSolrClientFactory mocks = new MockSolrClientFactory();
    static List<String> endpoints = ImmutableList.of("foo", "bar");

    @BeforeClass
    public static void before() throws Throwable
    {
        Clients.SOLR_SERVER_FACTORY = mocks;
    }

    @Before
    public void setUp() throws Throwable
    {
        Clients.reset();
    }

    @After
    public void tearDown() throws Throwable
    {
        Iterator<Callable> i = SolrStress.shutdownTasks.descendingIterator();
        while (i.hasNext())
        {
            i.next().call();
        }
        mocks.reset();
    }

    /*
     * Test a file can be loaded via HTTP
     */
    @Test
    public void testHTTPLoad() throws Throwable
    {
        // given:
        File geonames = GeonamesReaderTest.getClasspathResource("geonames.txt");

        // when:
        try (GeonamesLoader reader = new GeonamesLoader(geonames.toPath(), Integer.MAX_VALUE, ImmutableMap.of(), Suppliers.ofInstance(new Random(42))))
        {
            CommandLine.Params params = new CommandLine.Params();
            params.urls = ImmutableSet.of("foo");
            Metrics metrics = new Metrics(1, TimeUnit.HOURS, 0, ImmutableList.<PrintStream>of(), Long.MAX_VALUE, null);

            HTTPLoadStrategy load = new HTTPLoadStrategy(params, metrics, reader, null);
            MockSolrClientFactory.MockSolrClient mock = mocks.mocks.get(0);
            mock.writePermits.release(50);
            load.execute();

            // then:
            assertEquals(50, mocks.pendingWrites());
        }

    }
}
